@@ -17,41 +17,41 @@ from .motion import Motion
 
 
 class _DroneStateWatcher(threading.Thread):
-    """Subscribes to UDP messages from the drone and stores the latest data
-    """
+    """Subscribes to UDP messages from the drone and stores the latest data"""
 
-    def __init__(self, ip="192.168.1.101"):
+    def __init__(self, ip: str = "192.168.1.101", udp_timeout: float = 3):
         threading.Thread.__init__(self)
         self._ip = ip
+        self._udp_timeout = udp_timeout
         self._general_state = None
+        self._general_state_received = threading.Event()
         self._calibration_state = None
-        self._udpclient = UdpClient(drone_ip=self._ip)
+        self._calibration_state_received = threading.Event()
+        self._udp_client = UdpClient(drone_ip=self._ip)
         self._exit_flag = threading.Event()
         self.daemon = True
 
     @property
     def general_state(self) -> dict:
-        start = time.time()
-        while self._general_state is None:
-            if time.time() - start > 3:
-                raise TimeoutError("No state message received from drone")
+        if not self._general_state_received.wait(timeout=self._udp_timeout):
+            raise TimeoutError("No state message received from drone")
         return self._general_state
 
     @property
     def calibration_state(self) -> dict:
-        start = time.time()
-        while self._calibration_state is None:
-            if time.time() - start > 3:
-                raise TimeoutError("No state message received from drone")
+        if not self._calibration_state_received.wait(timeout=self._udp_timeout):
+            raise TimeoutError("No state message received from drone")
         return self._calibration_state
 
     def run(self):
         while not self._exit_flag.is_set():
-            data_packet = self._udpclient.get_data_dict()
+            data_packet = self._udp_client.get_data_dict()
             if data_packet["command_type"] == 1:
                 self._general_state = data_packet
+                self._general_state_received.set()
             elif data_packet["command_type"] == 2:
                 self._calibration_state = data_packet
+                self._calibration_state_received.set()
 
     def stop(self):
         self._exit_flag.set()
@@ -107,11 +107,7 @@ class Config:
 
     @water_density.setter
     def water_density(self, density: int):
-        if version.parse(self._parent_drone.software_version_short) < version.parse("1.5"):
-            raise RuntimeError(
-                "Drone software version is too old. Setting water density requires version 1.5"
-                " or higher."
-            )
+        self._parent_drone._verify_required_blunux_version("1.5")
         self._water_density = density
         self._parent_drone._tcp_client.set_water_density(density)
 
@@ -134,15 +130,22 @@ class Drone:
     `slaveModeEnabled=True` you will still be able to receive data from the drone.
     """
 
-    def __init__(self, ip="192.168.1.101", tcpPort=2011, autoConnect=True, slaveModeEnabled=False):
+    def __init__(
+        self,
+        ip="192.168.1.101",
+        tcpPort=2011,
+        autoConnect=True,
+        slaveModeEnabled=False,
+        udpTimeout=3,
+    ):
         self._ip = ip
         self._port = tcpPort
-        self._slaveModeEnabled = slaveModeEnabled
+        self._slave_mode_enabled = slaveModeEnabled
         if slaveModeEnabled:
             self._tcp_client = _SlaveTcpClient()
         else:
             self._tcp_client = _NoConnectionTcpClient()
-        self._state_watcher = _DroneStateWatcher(ip=self._ip)
+        self._state_watcher = _DroneStateWatcher(ip=self._ip, udp_timeout=udpTimeout)
         self.camera = Camera(self)
         self.motion = Motion(self)
         self.logs = Logs(self)
@@ -151,6 +154,26 @@ class Drone:
         if autoConnect is True:
             self.connect(timeout=3)
 
+    def _verify_required_blunux_version(self, requirement: str):
+        """Verify that Blunux version is higher than requirement
+
+        requirement needs to be a string that's able to be parsed by version.parse()
+
+        Raises a RuntimeError if the Blunux version of the connected drone does not match or exceed
+        the requirement.
+        """
+
+        if not self.connection_established:
+            raise ConnectionError(
+                "The connection to the drone is not established, try calling the connect method "
+                "before retrying"
+            )
+        if version.parse(self.software_version_short) < version.parse(requirement):
+            raise RuntimeError(
+                f"Blunux version of connected drone is {self.software_version_short}. Version "
+                f"{requirement} or higher is required."
+            )
+
     @property
     def connection_established(self):
         if isinstance(self._tcp_client, _NoConnectionTcpClient):
@@ -180,7 +203,7 @@ class Drone:
         self.uuid = response["hardware_id"]
 
     @staticmethod
-    def _wait_for_udp_communication(timeout: int, ip="192.168.1.101"):
+    def _wait_for_udp_communication(timeout: float, ip: str = "192.168.1.101"):
         """Simple helper for waiting for drone to come online
 
         Raises ConnectionError if no connection is established in the specified timeout.
@@ -221,16 +244,19 @@ class Drone:
             # Ignore multiple starts
             pass
 
-    def connect(self, timeout=None):
+    def connect(self, timeout: float = None):
         """Start receiving telemetry info from the drone, and publishing watchdog messages
 
         When watchdog message are published the thrusters are armed, to stop the drone from moving
         unexpectedly when connecting all thruster set points are set to zero when connecting.
+
+        - *timeout* (float): Seconds to wait for connection
         """
+
         self._wait_for_udp_communication(timeout, self._ip)
         self._update_drone_info()
         self._start_state_watcher_thread()
-        if self._slaveModeEnabled:
+        if self._slave_mode_enabled:
             # No need to touch the TCP stuff if we're in slave mode so we return early
             return
 
@@ -265,7 +291,7 @@ class Drone:
 
     def disconnect(self):
         """Disconnects the TCP connection, allowing another client to take control of the drone"""
-        if self.connection_established and not self._slaveModeEnabled:
+        if self.connection_established and not self._slave_mode_enabled:
             self._clean_up_tcp_client()
 
     @property
@@ -298,7 +324,6 @@ class Drone:
 
         * depth (int): The depth in millimeters of water column.
         """
-
         return self._state_watcher.general_state["depth"]
 
     @property
@@ -326,17 +351,32 @@ class Drone:
         """
         return self._state_watcher.general_state["battery_state_of_charge_rel"]
 
+    @property
+    def error_flags(self) -> int:
+        """Get the error flags
+
+        *Returns*:
+
+        * error_flags (int): The error flags as int
+        """
+        return self._state_watcher.general_state["error_flags"]
+
+    @property
+    def active_video_streams(self) -> int:
+        """Get the number of currently active connections to the video stream
+
+        Every client connected to the RTSP stream (does not matter if it's directly from GStreamer,
+        or from the Blueye app) counts as one connection.
+
+        Requires Blunux version 1.5.33 or newer.
+        """
+
+        self._verify_required_blunux_version("1.5.33")
+        SPECTATORS_MASK = 0x000000FF00000000
+        SPECTATORS_OFFSET = 32
+        debug_flags = self._state_watcher._general_state["debug_flags"]
+        return (debug_flags & SPECTATORS_MASK) >> SPECTATORS_OFFSET
+
     def ping(self):
         """Ping drone, an exception is thrown by TcpClient if drone does not answer"""
         self._tcp_client.ping()
-
-
-class Pioneer(Drone):
-    def __init__(self, *args, **kwargs):
-        warnings.warn(
-            "This class has been deprecated and will be removed in the next major version, "
-            "please use the Drone class instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        super().__init__(*args, **kwargs)
